var db;
var algorithms = {
    symmetric_: {
      name: 'AES-CBC',
      length: 128,
      // Temporarily zero-out 64 bit initialization vector.
      // This should be changed to something more secure
      // in the future.
      iv: new Uint8Array(16)
    },
    asymmetric_: {
      name: 'RSASSA-PKCS1-v1_5',
      modulusLength: 2048,
      // Equivalent to 65537
      publicExponent: new Uint8Array([0x01, 0x00, 0x01]),
      hash: { name: 'SHA-256' }
    }
  };


function openDB(){
  return new Promise(function(resolve, reject) {
    var request = window.indexedDB.open('keystore', 1);

    request.onerror = function(event) {
      reject(new Error(event.target.error.message));
    };

    request.onsuccess = function(event) {
      db = event.target.result;
      resolve();
    };

    request.onupgradeneeded = function(event) {
      
      db = event.target.result;

      // Create an object store called name_ with the autoIncrement flag set as true.
      var objectStore = db.createObjectStore('keys', { autoIncrement: true });

      objectStore.transaction.oncomplete = function(event) {
        resolve();
      };
    };
  });
}

function putKey(key){
  return new Promise(function(resolve, reject) {
    var transaction = db.transaction('keys', 'readwrite'),
          objectStore = transaction.objectStore('keys')
            .put(key, key.type);

      transaction.onabort = function(event) { reject(Error(event.target.error.message)); };
      transaction.onerror = function(event) { reject(Error(event.target.error.message)); };

      transaction.oncomplete = function(event) {
        console.log('successfully added key');
        resolve(key);
      };
  });
};

function listKeys(){
  
  return new Promise(function(resolve, reject) {
    if (!db) reject(new Error('KeyStore is not open.'));

    var list = [],
        cursor = db
          .transaction('keys', 'readonly')
          .objectStore('keys')
          .openCursor();

    cursor.onerror = function(event) { reject(Error(event.target.error.message)); };
    cursor.onabort = function(event) { reject(Error(event.target.error.message)); };

    cursor.onsuccess = function(event) {
      var result = event.target.result;
      if (result) {
        list.push({ id: result.key, value: result.value });
        result.continue();
      } else {
        resolve(list);
      }
    };
  });
  
}

function purge(){
  return new Promise(function(resolve, reject) {
    if (!db) reject(new Error('KeyStore is not open.'));

    var failed = false,
        cursor = db
          .transaction('keys', 'readwrite')
          .objectStore('keys')
          .openCursor();

    cursor.onerror = function(event) { reject(Error(event.target.error.message)); };
    cursor.onabort = function(event) { reject(Error(event.target.error.message)); };

    cursor.onsuccess = function(event) {
      var result = event.target.result;
      if (result) {
        var request = result.delete();

        request.onsuccess = function(event) {
          console.log('successfully purged key');
        };

        request.onerror = function(event) {
          console.error('failed to purge key');
          failed = true;
        };

        result.continue();
      } else {
        if (failed) {
          reject(Error('failed to purge one or more keys'));
        }
        else {
          resolve();
        }
      }
    };
  });
}

function generateKey(algo, extractable, keyUsages){
  return new Promise(function(resolve, reject) {
    window.crypto.subtle.generateKey(algo, extractable, keyUsages)
      .then(function(key) {
        console.log('successfully generated key');
        resolve(key);
      }, function(error) {
        console.log('failed to generate key');
        reject(error);
      });
  });
}


var keysToPut = [];

openDB()
.then(function(){
  return purge();
})
.then(function(){
  return listKeys();
})
.then(function(keys){
  console.log(keys);
  return generateKey(algorithms.symmetric_, true, ['encrypt', 'decrypt']);
})
.then(function(key){
  keysToPut.push(key);
  return generateKey(algorithms.asymmetric_, true, ['sign', 'verify']);
})
.then(function(key){
  keysToPut.push(key.privateKey);
  return putKey(keysToPut[0]);
})
.then(function(key){
  return putKey(keysToPut[1]);
})
.catch(function(error){
  console.log('errored out');
  console.log(error);
})
                     
                    
                     